//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;

import java.util.Observable;

import android.os.SystemClock;

public abstract class GameModel extends Observable {
	
	protected class UpdateRunnable implements Runnable {
		
		protected static final long TIME_BETWEEN_TICKS = 16; // ~60 FPS
		
		@Override
		public void run() {
			long curr_time = SystemClock.uptimeMillis();
			long accumulator = 0;
			
			while (!_stopped) {
				long new_time = SystemClock.uptimeMillis();
				long dt = new_time - curr_time;
				curr_time = new_time;
				
				accumulator += dt;
				
				while (accumulator >= TIME_BETWEEN_TICKS) {
					_game_time += TIME_BETWEEN_TICKS;
					onUpdate();
					accumulator -= TIME_BETWEEN_TICKS;
				}
				
				long update_time = SystemClock.uptimeMillis() - curr_time;
				if (update_time < TIME_BETWEEN_TICKS) {
					_running_slow = false;
					SystemClock.sleep(TIME_BETWEEN_TICKS - update_time);
				} else {
					_running_slow = true;
					Thread.yield(); //Still let the draw thread do it's work
				}
					
			}
		}
	}

	protected Thread _update_thread;     //Thread that generates onUpdate event
	protected boolean _running = false;  //Model has been started (but may be paused)
	protected boolean _stopped = false;  //Update thread should stop
	
	protected long _game_time = 0;
	protected boolean _running_slow = false;
	
	public abstract void onUpdate();
	public abstract InputSystem getInputSystem();
	
	public void start() {
		if (!isRunning()) {
			_running = true;
			_stopped = false;
			_update_thread = new Thread(new UpdateRunnable());
			_update_thread.start();
		}
	}
	
	public void stop() {
		_running = false;
		if (_update_thread.isAlive()) {
			_stopped = true;
			try {
				_update_thread.join();
			} catch (InterruptedException e) {
				//Ignore exception
			}
		}
	}
	
	public boolean isRunning() {
		return _running;
	}
	
	public void pause() {
		if (isRunning() && _update_thread.isAlive()) {
			_stopped = true;
			try {
				_update_thread.join();
			} catch (InterruptedException e) {
				//Ignore exception
			}
		}
	}
	
	public void resume() {
		if (isPaused()) {
			_stopped = false;
			_update_thread = new Thread(new UpdateRunnable());
			_update_thread.start();
		}
	}
	
	public boolean isPaused() {
		return (_running && !_update_thread.isAlive());
	}
	
	public long getGameTime() {
		return _game_time;
	}
	
	public boolean isRunningSlow() {
		return _running_slow;
	}
	
	public void onChange(Object data) {
		setChanged();
		notifyObservers(data);
	}
}
