//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;

import java.util.ArrayList;

import android.content.SharedPreferences;


public class PongGameModel extends GameModel {
	
	protected PongSettings _settings;
	protected PongInputSystem _input;
	protected ArrayList<PongPaddleModel> _paddles;
	protected ArrayList<PongPlayer> _players;
	protected PongBallModel _ball;
	protected int _width;
	protected int _height;
	protected int _wait_ticks = 0;
	
	public PongGameModel(SharedPreferences preferences, int width, int height) {
		_width = width;
		_height = height;
		 
		_settings = new PongSettings(preferences);
		_input = new PongInputSystem(this);
		
		_paddles = new ArrayList<PongPaddleModel>();
		_paddles.add(new PongPaddleModel(this, 0.1f * width, 0.5f * height, 0.02f * width, 0.2f * height));
		_paddles.add(new PongPaddleModel(this, 0.9f * width, 0.5f * height, 0.02f * width, 0.2f * height));
		
		_players = new ArrayList<PongPlayer>();
		_players.add(new PongPlayerAI(this, 0));
		_players.add(new PongPlayerAI(this, 1));
		
		_ball = new PongBallModel(this, 0.5f * width, 0.5f * height, 0.03f * height, 0.0f, 0.0f);
		startWait();
	}
	
	@Override
	public void start() {
		super.start();
		for (PongPlayer player : _players) {
			player.setPoints(0);
		}
		for (PongPaddleModel paddle : _paddles) {
			paddle.setY(0.5f * _height);
			paddle.setVelocity(0.0f, 0.0f);
		}
		resetBall();
		startWait();
		updateSettings();
	}
	
	@Override
	public void resume() {
		super.resume();
		updateSettings();
	}

	@Override
	public void onUpdate() {
		for (PongPlayer player : _players) {
			player.update();
		}
		for (PongPaddleModel paddle : _paddles) {
			paddle.update();
		}
		if (_wait_ticks > 0) {
			_wait_ticks -= 1;
			if (_wait_ticks == 0) {
				resetBall();
			}
		} else {
			_ball.update();
		}
		_input.update();
	}

	@Override
	public InputSystem getInputSystem() {
		return _input;
	}
	
	public PongSettings getSettings() {
		return _settings;
	}
	
	public PongPaddleModel getPaddle(int index) {
		return _paddles.get(index);
	}
	
	public PongPlayer getPlayer(int index) {
		return _players.get(index);
	}
	
	public PongBallModel getBall() {
		return _ball;
	}
	
	public int getWidth() {
		return _width;
	}
	
	public int getHeight() {
		return _height;
	}
	
	public void onScore(float x, float y) {
		if (x <= _ball.getSize() / 2.0f) {
			_players.get(1).addPoint();
		} else if (x >= _width - (_ball.getSize() / 2.0f)) {
			_players.get(0).addPoint();
		}
		if (_players.get(0).getPoints() == _settings.getMaxPoints() ||
				_players.get(1).getPoints() == _settings.getMaxPoints()) {
			stop();
			return;
		}
		_ball.setPos(0.5f * _width, 0.5f * _height);
		startWait();
	}
	
	protected void startWait() {
		_wait_ticks = 60;
	}
	
	protected void updateSettings() {
		int points = _players.get(0).getPoints();
		if (_settings.getPlayer1Input() == PongSettings.INPUT_AI) {
			_players.set(0, new PongPlayerAI(this, 0));
			_players.get(0).setPoints(points);
		} else {
			_players.set(0, new PongPlayerHuman(this, 0, _settings.getPlayer1Input()));
			_players.get(0).setPoints(points);			
		}
		points = _players.get(1).getPoints();
		if (_settings.getPlayer2Input() == PongSettings.INPUT_AI) {
			_players.set(1, new PongPlayerAI(this, 1));
			_players.get(1).setPoints(points);
		} else {
			_players.set(1, new PongPlayerHuman(this, 1, _settings.getPlayer2Input()));
			_players.get(1).setPoints(points);			
		}
	}
	
	protected void resetBall() {
		_ball.setPos(0.5f * _width, 0.5f * _height);
		float vx = 0.0f;
		float vy = 0.0f;
		if (Math.random() < 0.5) {
			vx = _width * (0.006f + (float)Math.random() * 0.004f);
		} else {
			vx = -_width * (0.006f + (float)Math.random() * 0.004f);
		}
		if (Math.random() < 0.5) {
			vy = _height * (float)Math.random() * 0.0015f;
		} else {
			vy = -_height * (float)Math.random() * 0.0015f;
		}
		_ball.setVelocity(vx, vy);
	}
}
