//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.model;

import android.view.KeyEvent;
import android.view.MotionEvent;

public class PongInputSystem implements InputSystem {
	
	protected PongGameModel _model;
	protected int _key_dir = DIRECTION_NONE;
	protected float _ball_dir = DIRECTION_NONE;
	protected float _left_touch_y = -1.0f;
	protected float _right_touch_y = -1.0f;
	
	protected final float TRACKBALL_SLOWDOWN_FACTOR = 0.9f;
	protected final int TOUCH_MOVEMENT_DELTA = 3;
	
	public static final int DIRECTION_NONE = 0;
	public static final int DIRECTION_UP = 1;
	public static final int DIRECTION_DOWN = -1;
	
	protected void setTouchValue(float x, float y) {
		if (x < _model.getWidth() / 2.0f) {
			_left_touch_y = y;
		} else {
			_right_touch_y = y;
		}
	}
	
	public PongInputSystem(PongGameModel model) {
		_model = model;
	}

	@Override
	public void handleKeyEvent(KeyEvent event) {
		switch (event.getKeyCode()) {
		case KeyEvent.KEYCODE_DPAD_UP:
			if (event.getAction() == KeyEvent.ACTION_DOWN) {
				_key_dir = DIRECTION_UP;
			} else if (event.getAction() == KeyEvent.ACTION_UP &&
					_key_dir == DIRECTION_UP) {
				_key_dir = DIRECTION_NONE;
			}
			break;
		case KeyEvent.KEYCODE_DPAD_DOWN:
			if (event.getAction() == KeyEvent.ACTION_DOWN) {
				_key_dir = DIRECTION_DOWN;
			} else if (event.getAction() == KeyEvent.ACTION_UP &&
					_key_dir == DIRECTION_DOWN) {
				_key_dir = DIRECTION_NONE;
			}
		}
	}

	@Override
	public void handleTouchEvent(MotionEvent event) {
		for (int i=0; i < event.getPointerCount(); i++) {
			setTouchValue(event.getX(i), event.getY(i));
		}
		if (event.getActionMasked() == MotionEvent.ACTION_UP || 
			event.getActionMasked() == MotionEvent.ACTION_POINTER_UP) {
			setTouchValue(event.getX(event.getActionIndex()), -1.0f);
		}
	}

	@Override
	public void handleTrackballEvent(MotionEvent event) {
		_ball_dir += event.getY() * -1.0f;
	}
	
	@Override
	public void update() {
		_ball_dir *= TRACKBALL_SLOWDOWN_FACTOR;
	}
	
	public float getKeyboardMovement() {
		return _key_dir;
	}
	
	public float getTouchMovement(float reference_y, int paddle_index) {
		float value = 0.0f;
		if (paddle_index == 0) {
			value = _left_touch_y;
		} else if (paddle_index == 1) {
			value = _right_touch_y;
		}
		if (value >= 0.0f) {
			if ((_model.getHeight() - value) > reference_y + TOUCH_MOVEMENT_DELTA) {
				return DIRECTION_UP;
			} else if ((_model.getHeight() - value) < reference_y - TOUCH_MOVEMENT_DELTA) {
				return DIRECTION_DOWN;
			}
		}
		return DIRECTION_NONE;
	}
	
	public float getTrackballMovement() {
		return _ball_dir;
	}

}
