//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;

import java.util.ArrayList;
import java.util.Observable;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import javax.microedition.khronos.opengles.GL11;

import android.content.Context;
import android.graphics.PointF;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;
import aschiefer.bouncy.R;
import aschiefer.bouncy.model.GameModel;
import aschiefer.bouncy.model.PongGameModel;


public class PongGameRenderer implements GameRenderer {
	
	protected Context _context;
	protected ViewGroup _layout;
	protected PongGameModel _model;
	protected PongBackgroundRenderer _background;
	protected ArrayList<PongPaddleRenderer> _paddles;
	protected PongBallRenderer _ball;
	protected TextView _score0;
	protected TextView _score1;
	protected ResourceManager _resources;
	protected int _sound_collision;
	protected boolean _finished;
	
	public PongGameRenderer(Context context) {
		_context = context;
		_resources = new ResourceManager(context);
		_sound_collision = _resources.loadSound(R.raw.pong);
		_finished = false;
	}

	@Override
	public void setLayout(ViewGroup layout) {
		_layout = layout;
		_score0 = (TextView) _layout.findViewById(R.id.TextView01);
		_score1 = (TextView) _layout.findViewById(R.id.TextView03);
	}
	
	@Override
	public void setGameModel(GameModel model)
	{
		_model = (PongGameModel)model;
		_model.addObserver(this);
		_background = new PongBackgroundRenderer(_model, _resources);
		_paddles = new ArrayList<PongPaddleRenderer>();
		_paddles.add(new PongPaddleRenderer(_model, _resources, 0));
		_paddles.add(new PongPaddleRenderer(_model, _resources, 1));
		_ball = new PongBallRenderer(_model, _resources);
	}
	
	@Override
	public void onDrawFrame(GL10 gl) {		
		GL11 gl11 = (GL11)(gl);
		
		if (_model.getSettings().isDebug() && _model.isRunningSlow()) {
			gl11.glClearColor(1.0f, 0.0f, 0.0f, 1.0f);
		} else {
			gl11.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		}
		if (!_finished) {
			if (_model.getPlayer(0).getPoints() == _model.getSettings().getMaxPoints()) {
				_finished = true;
				_layout.post(new Runnable() {
					@Override
					public void run() {
						_layout.findViewById(R.id.ButtonGameStart).setVisibility(View.VISIBLE);
						Toast.makeText(_context, R.string.player1_won, Toast.LENGTH_LONG).show();
					}			
				});
			}
			if (_model.getPlayer(1).getPoints() == _model.getSettings().getMaxPoints()) {
				_finished = true;
				_layout.post(new Runnable() {
					@Override
					public void run() {
						_layout.findViewById(R.id.ButtonGameStart).setVisibility(View.VISIBLE);
						Toast.makeText(_context, R.string.player2_won, Toast.LENGTH_LONG).show();
					}			
				});
			}
		} else {
			if (_model.isRunning()) {
				_finished = false;
			}
		}
		_layout.post(new Runnable() {
			@Override
			public void run() {
				_score0.setText(Integer.toString(_model.getPlayer(0).getPoints()));
				_score1.setText(Integer.toString(_model.getPlayer(1).getPoints()));
			}			
		});
		
		gl11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

		gl.glMatrixMode(GL11.GL_TEXTURE);
		gl.glLoadIdentity();
		gl.glScalef(1.0f, -1.0f, 1.0f);
		gl.glMatrixMode(GL11.GL_MODELVIEW);
		gl.glDisable(GL11.GL_TEXTURE_2D);
		
		_resources.bindMeshData(gl11);	
		
		gl11.glLoadIdentity();
		
		_background.draw(gl11);
		
		for (PongPaddleRenderer paddle : _paddles) {
			gl11.glLoadIdentity();
			paddle.draw(gl11);
		}
		gl11.glLoadIdentity();
		_ball.draw(gl11);
	}

	@Override
	public void onSurfaceChanged(GL10 gl, int w, int h) {
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glViewport(0, 0, w, h);
		gl.glOrthof(0.0f, w, 0.0f, h, 0.0f, 1.0f);
		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig conf) {
		GL11 gl11 = (GL11)(gl);
		_resources.clearCache();
		setup(gl11);
		_resources.loadMeshes(gl11);
		
		gl11.glDisable(GL11.GL_DITHER);
		gl11.glShadeModel(GL11.GL_SMOOTH);
		gl11.glEnable(GL11.GL_DEPTH_TEST);

		gl11.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		gl11.glEnable(GL10.GL_BLEND);
		
		gl11.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	}

	@Override
	public void update(Observable observable, Object data) {
		if (_model.getSettings().playSound()) {
			float left = ((PointF)data).x / _model.getWidth();
			float right = 1.0f - left;
			_resources.playSound(_sound_collision, left, right);
		}
	}

	
	protected void setup(GL11 gl) {
		_background.setup(gl);
		for (PongPaddleRenderer paddle : _paddles) {
			paddle.setup(gl);
		}
		_ball.setup(gl);
	}
}
