//------------------------------------------------------------------------------
//Copyright (c) 2010, Andreas Schiefer
//All rights reserved.
//
//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, 
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//    * Neither the name of the Bouncy nor the names of its contributors 
//      may be used to endorse or promote products derived from this software
//      without specific prior written permission.
//
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
//SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
//CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------

package aschiefer.bouncy.renderer;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.microedition.khronos.opengles.GL11;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.AudioManager;
import android.media.SoundPool;
import android.opengl.GLUtils;

public class ResourceManager {
	
	protected Context _context;
	protected SoundPool _sounds;
	protected VertexBuffer _vertices;
	protected Set<Integer> _requested_meshes;
	protected List<MeshInfo> _mesh_data;
	protected HashMap<Integer, Integer> _tex_cache;
	
	public ResourceManager(Context context) {
		_context = context;
		_sounds = new SoundPool(2, AudioManager.STREAM_MUSIC, 0);
		_requested_meshes = new HashSet<Integer>();
		_mesh_data = new ArrayList<MeshInfo>();
		_tex_cache = new HashMap<Integer, Integer>();
	}
	
	public void clearCache() {
		_tex_cache.clear();
		_mesh_data.clear();
		_vertices = null;
	}
	
	public int loadSound(int res_id) {
		return _sounds.load(_context, res_id, 1);
	}
	
	public int playSound(int sound_id, float left, float right) {
		return _sounds.play(sound_id, left, right, 0, 0, 1.0f);
	}
	
	public void requestMesh(int mesh_id) {
		_requested_meshes.add(mesh_id);
	}
	
	public void loadMeshes(GL11 gl) {
		_vertices = new VertexBuffer(gl);
		for (int mesh_id : _requested_meshes) {
			MeshInfo mesh = MeshInfo.getFromID(mesh_id);
			_mesh_data.add(mesh);
		}
		_vertices.addObjects(gl, _mesh_data.toArray(new MeshInfo[0]));
	}
	
	public void bindMeshData(GL11 gl) {
		_vertices.bind(gl);
	}
	
	public void drawMesh(GL11 gl, int mesh_id) {
		MeshInfo.getFromID(mesh_id).draw(gl);
	}
	
	public int getTextureID(GL11 gl, int texture) {
		if (_tex_cache.containsKey(texture)) {
			return _tex_cache.get(texture);
		}
		InputStream is = _context.getResources().openRawResource(texture);
	    Bitmap tex_data = null;
	    try {
	        tex_data = BitmapFactory.decodeStream(is);
	    } finally {
	        //Always clear and close
	        try {
	            is.close();
	            is = null;
	        } catch (IOException e) {
	        }
	    }		
		int[] tex_id = new int[1];
		gl.glGenTextures(1, tex_id, 0);
		gl.glBindTexture(GL11.GL_TEXTURE_2D, tex_id[0]);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR_MIPMAP_NEAREST);
		gl.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_GENERATE_MIPMAP, GL11.GL_TRUE);
		GLUtils.texImage2D(GL11.GL_TEXTURE_2D, 0, tex_data, 0);
		tex_data.recycle();
		_tex_cache.put(texture, tex_id[0]);
		return tex_id[0];
	}
}
